/*
    tstore - an archive format for archiving tar archives
    Copyright (C) 2004-2005  Kasper Dupont

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    with permission from the author any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    $Id: tcat.c,v 1.4 2005/04/24 14:09:49 kasperd Exp $
*/

#include "tstore.h"

uint8_t pointers[8][512];

int main(int argc, char ** argv)
{
  struct store *s;
  uint8_t *buf;
  uint64_t idx,nidx;
  uint64_t i;

  if (argc != 3) {
    fprintf(stderr,"Uage: %s <store> <file>\n",argv[0]);
    return 1;
  }

  s=open_store(argv[1],O_RDONLY,DONT_LOCK);
  if (!s) errstop(argv[1]);
  buf=s->header_buf;

  for (idx=GET_BNUM(buf+NEXTPTR_OFFSET); idx; idx=nidx) {
    get_block(s,idx,buf);
    if (!strcmp(argv[2],buf+FILENAME_OFFSET)) {
      uint64_t size=GET_SIZE(buf+FILESIZE_OFFSET);
      uint64_t directory=GET_BNUM(buf+TREEPTR_OFFSET);
      int levels=filelevels(size);

      for (i=0;i<(size+511)/512;++i) {
	int j;
	int siz;
	uint64_t block=directory;
	for (j=levels-1;j>=0;--j) {
	  int idx,k;
	  idx=i;
	  for (k=0;k<j;++k) idx/=102;
	  get_block(s,block,(void*)pointers[j]);
	  block=GET_BNUM(pointers[j]+(idx%102)*5);
	}
	get_block(s,block,buf);
	siz=512;
	if (i==size/512) siz=size%512;
	assert(write(1,buf,siz)==siz);
      }
      return 0;
    }
    nidx=GET_BNUM(buf+NEXTPTR_OFFSET);
    if ((nidx>=s->store_size-1)||(nidx==idx)) {
      fprintf(stderr,"Invalid link %lld=>%lld\n",idx,nidx);
      return 1;
    }
  }
  fprintf(stderr,"%s: not found\n",argv[2]);
  return 1;
}
